@@ -1,9 +1,11 @@
 """JSON API router class."""
+from enum import Enum, auto
 from inspect import Parameter, Signature, signature
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
+    ClassVar,
     Dict,
     Iterable,
     List,
@@ -14,7 +16,7 @@ from typing import (
     Union,
 )
 
-from fastapi import APIRouter, Path, Query, Request, status
+from fastapi import APIRouter, Body, Path, Query, Request, status
 from pydantic import BaseModel as PydanticBaseModel
 
 from fastapi_jsonapi.data_typing import TypeModel
@@ -22,6 +24,7 @@ from fastapi_jsonapi.exceptions import ExceptionResponseSchema
 from fastapi_jsonapi.schema_base import BaseModel
 from fastapi_jsonapi.schema_builder import SchemaBuilder
 from fastapi_jsonapi.signature import create_additional_query_params
+from fastapi_jsonapi.utils.dependency_helper import DependencyHelper
 from fastapi_jsonapi.views.utils import (
     HTTPMethod,
     HTTPMethodConfig,
@@ -39,8 +42,24 @@ JSONAPIObjectSchemaType = TypeVar("JSONAPIObjectSchemaType", bound=PydanticBaseM
 not_passed = object()
 
 
+class ViewMethods(str, Enum):
+    GET_LIST = auto()
+    POST = auto()
+    DELETE_LIST = auto()
+    GET = auto()
+    DELETE = auto()
+    PATCH = auto()
+
+
 class RoutersJSONAPI:
-    """API Router interface for JSON API endpoints in web-services."""
+    """
+    API Router interface for JSON API endpoints in web-services.
+    """
+
+    # xxx: store in app, not in routers!
+    all_jsonapi_routers: ClassVar[Dict[str, "RoutersJSONAPI"]] = {}
+    Methods = ViewMethods
+    DEFAULT_METHODS = tuple(str(method) for method in ViewMethods)
 
     def __init__(
         self,
@@ -58,6 +77,7 @@ class RoutersJSONAPI:
         pagination_default_number: Optional[int] = 1,
         pagination_default_offset: Optional[int] = None,
         pagination_default_limit: Optional[int] = None,
+        methods: Iterable[str] = (),
     ) -> None:
         """
         Initialize router items.
@@ -90,11 +110,18 @@ class RoutersJSONAPI:
         self.list_views = None
         self.detail_view_resource: Type["DetailViewBase"] = class_detail
         self.list_view_resource: Type["ListViewBase"] = class_list
-        self._type: str = resource_type
+        self.type_: str = resource_type
         self._schema: Type[BaseModel] = schema
         self.schema_list: Type[BaseModel] = schema
         self.model: Type[TypeModel] = model
         self.schema_detail = schema
+        # tuple and not set, so ordering is persisted
+        self.methods = tuple(methods) or self.DEFAULT_METHODS
+
+        if self.type_ in self.all_jsonapi_routers:
+            msg = f"Resource type {self.type_!r} already registered"
+            raise ValueError(msg)
+        self.all_jsonapi_routers[self.type_] = self
 
         self.pagination_default_size: Optional[int] = pagination_default_size
         self.pagination_default_number: Optional[int] = pagination_default_number
@@ -107,8 +134,18 @@ class RoutersJSONAPI:
             schema_in_post=schema_in_post,
             schema_in_patch=schema_in_patch,
         )
-        self._schema_in_post = dto.schema_in_post
-        self._schema_in_patch = dto.schema_in_patch
+        # we need to save post_data and patch_data
+        # and set dependency `data` as `embed=True`
+        # because if there's more than one Body dependency,
+        # FastAPI makes them all `embed=True` and validation breaks!
+        # doc url
+        # https://fastapi.tiangolo.com/tutorial/body-multiple-params/#embed-a-single-body-parameter
+        # code:
+        # https://github.com/tiangolo/fastapi/blob/831b5d5402a65ee9f415670f4116522c8e874ed3/fastapi/dependencies/utils.py#L768
+        self.schema_in_post = dto.schema_in_post
+        self.schema_in_post_data = dto.schema_in_post_data
+        self.schema_in_patch = dto.schema_in_patch
+        self.schema_in_patch_data = dto.schema_in_patch_data
         self.detail_response_schema = dto.detail_response_schema
         self.list_response_schema = dto.list_response_schema
 
@@ -136,11 +173,13 @@ class RoutersJSONAPI:
         kind: Literal["list", "detail"],
     ):
         """
+        Generate view name
+
         :param action
         :param kind: list / detail
         :return:
         """
-        return f"{action}_{self._type}_{kind}"
+        return f"{action}_{self.type_}_{kind}"
 
     def _register_get_resource_list(self, path: str):
         list_response_example = {
@@ -151,7 +190,7 @@ class RoutersJSONAPI:
             tags=self._tags,
             responses=list_response_example | self.default_error_responses,
             methods=["GET"],
-            summary=f"Get list of `{self._type}` objects",
+            summary=f"Get list of `{self.type_}` objects",
             endpoint=self._create_get_resource_list_view(),
             name=self.get_endpoint_name("get", "list"),
         )
@@ -165,7 +204,7 @@ class RoutersJSONAPI:
             tags=self._tags,
             responses=create_resource_response_example | self.default_error_responses,
             methods=["POST"],
-            summary=f"Create object `{self._type}`",
+            summary=f"Create object `{self.type_}`",
             status_code=status.HTTP_201_CREATED,
             endpoint=self._create_post_resource_list_view(),
             name=self.get_endpoint_name("create", "list"),
@@ -180,7 +219,7 @@ class RoutersJSONAPI:
             tags=self._tags,
             responses=detail_response_example | self.default_error_responses,
             methods=["DELETE"],
-            summary=f"Delete objects `{self._type}` by filters",
+            summary=f"Delete objects `{self.type_}` by filters",
             endpoint=self._create_delete_resource_list_view(),
             name=self.get_endpoint_name("delete", "list"),
         )
@@ -196,13 +235,13 @@ class RoutersJSONAPI:
             tags=self._tags,
             responses=detail_response_example | self.default_error_responses,
             methods=["GET"],
-            summary=f"Get object `{self._type}` by id",
+            summary=f"Get object `{self.type_}` by id",
             endpoint=self._create_get_resource_detail_view(),
             name=self.get_endpoint_name("get", "detail"),
         )
 
     def _register_patch_resource_detail(self, path: str):
-        detail_response_example = {
+        update_response_example = {
             status.HTTP_200_OK: {"model": self.detail_response_schema},
         }
         self._router.add_api_route(
@@ -210,27 +249,31 @@ class RoutersJSONAPI:
             # TODO: trailing slash (optional)
             path=path + "/{obj_id}",
             tags=self._tags,
-            responses=detail_response_example | self.default_error_responses,
+            responses=update_response_example | self.default_error_responses,
             methods=["PATCH"],
-            summary=f"Patch object `{self._type}` by id",
+            summary=f"Patch object `{self.type_}` by id",
             endpoint=self._create_patch_resource_detail_view(),
             name=self.get_endpoint_name("update", "detail"),
         )
 
     def _register_delete_resource_detail(self, path: str):
-        detail_response_example = {
-            status.HTTP_200_OK: {"model": self.detail_response_schema},
+        delete_response_example = {
+            status.HTTP_204_NO_CONTENT: {
+                "description": "If a server is able to delete the resource,"
+                " the server MUST return a result with no data",
+            },
         }
         self._router.add_api_route(
             # TODO: variable path param name (set default name on DetailView class)
             # TODO: trailing slash (optional)
             path=path + "/{obj_id}",
             tags=self._tags,
-            responses=detail_response_example | self.default_error_responses,
+            responses=delete_response_example | self.default_error_responses,
             methods=["DELETE"],
-            summary=f"Delete object `{self._type}` by id",
+            summary=f"Delete object `{self.type_}` by id",
             endpoint=self._create_delete_resource_detail_view(),
             name=self.get_endpoint_name("delete", "detail"),
+            status_code=status.HTTP_204_NO_CONTENT,
         )
 
     def _create_pagination_query_params(self) -> List[Parameter]:
@@ -394,9 +437,58 @@ class RoutersJSONAPI:
 
         return self._create_dependency_params_from_pydantic_model(method_config.dependencies)
 
+    def prepare_dependencies_handler_signature(
+        self,
+        custom_handler: Callable[..., Any],
+        method_config: HTTPMethodConfig,
+    ) -> Signature:
+        sig = signature(custom_handler)
+
+        additional_dependency_params = []
+        if method_config.dependencies is not None:
+            additional_dependency_params = self._create_dependency_params_from_pydantic_model(
+                model_class=method_config.dependencies,
+            )
+
+        params, tail_params = self._get_separated_params(sig)
+
+        return sig.replace(parameters=params + list(additional_dependency_params) + tail_params)
+
+    async def handle_view_dependencies(
+        self,
+        request: Request,
+        view_cls: Type["ViewBase"],
+        method: HTTPMethod,
+    ) -> Dict[str, Any]:
+        """
+        Combines all dependencies (prepared) and returns them as list
+
+        Consider method config is already prepared for generic views
+        Reuse the same config for atomic operations
+
+        :param request:
+        :param view_cls:
+        :param method:
+        :return:
+        """
+        method_config: HTTPMethodConfig = view_cls.method_dependencies[method]
+
+        def handle_dependencies(**dep_kwargs):
+            return dep_kwargs
+
+        handle_dependencies.__signature__ = self.prepare_dependencies_handler_signature(
+            custom_handler=handle_dependencies,
+            method_config=method_config,
+        )
+
+        dep_helper = DependencyHelper(request=request)
+        dependencies_result: Dict[str, Any] = await dep_helper.run(handle_dependencies)
+        return dependencies_result
+
     def _create_get_resource_list_view(self):
         """
         Create wrapper for GET list (get objects list)
+
         :return:
         """
 
@@ -425,16 +517,21 @@ class RoutersJSONAPI:
 
         :return:
         """
-        schema_in = self._schema_in_post
+        # `data` as embed Body param
+        schema_in = self.schema_in_post_data
 
-        async def wrapper(request: Request, data_create: schema_in, **extra_view_deps):
+        async def wrapper(
+            request: Request,
+            data: schema_in = Body(embed=True),
+            **extra_view_deps,
+        ):
             resource = self.list_view_resource(
                 request=request,
                 jsonapi=self,
             )
 
             response = await resource.handle_post_resource_list(
-                data_create=data_create,
+                data_create=data,
                 **extra_view_deps,
             )
             return response
@@ -515,11 +612,12 @@ class RoutersJSONAPI:
 
         :return:
         """
-        schema_in = self._schema_in_patch
+        # `data` as embed Body param
+        schema_in = self.schema_in_patch_data
 
         async def wrapper(
             request: Request,
-            data_update: schema_in,
+            data: schema_in = Body(embed=True),
             obj_id: str = Path(...),
             **extra_view_deps,
         ):
@@ -531,7 +629,7 @@ class RoutersJSONAPI:
             # TODO: pass obj_id as kwarg (get name from DetailView class)
             response = await resource.handle_update_resource(
                 obj_id=obj_id,
-                data_update=data_update.data,
+                data_update=data,
                 **extra_view_deps,
             )
             return response
@@ -587,10 +685,19 @@ class RoutersJSONAPI:
         :param path:
         :return:
         """
-        self._register_get_resource_list(path)
-        self._register_post_resource_list(path)
-        self._register_delete_resource_list(path)
+        methods_map: Dict[Union[str, ViewMethods], Callable[[str], None]] = {
+            ViewMethods.GET_LIST: self._register_get_resource_list,
+            ViewMethods.POST: self._register_post_resource_list,
+            ViewMethods.DELETE_LIST: self._register_delete_resource_list,
+            ViewMethods.GET: self._register_get_resource_detail,
+            ViewMethods.PATCH: self._register_patch_resource_detail,
+            ViewMethods.DELETE: self._register_delete_resource_detail,
+        }
+        # patch for Python < 3.11
+        for key, value in list(methods_map.items()):
+            methods_map[str(key)] = value
 
-        self._register_get_resource_detail(path)
-        self._register_patch_resource_detail(path)
-        self._register_delete_resource_detail(path)
+        for method in self.methods:
+            # `to str` so Python < 3.11 is supported
+            register = methods_map[str(method)]
+            register(path)
